//
// ctrlPanel.cpp
//
// implementation for control panel class
//

#include "aniUI.h"
#include "ctrlPanel.h"

extern ctrlPanel *g_panel;
extern CAniUI* aniUI;

ctrlPanel::ctrlPanel(const char* title) : Fl_Window(100, 100, title), widgets() 
{ 
	m_nTargetHeight=WINDOW_HEIGHT; 
	m_nWidgetPerColumn=(m_nTargetHeight-INIT_SPACING)/YPIXELS_PER_WIDGET; 
	m_pObject=NULL;
	user_data(this);
	callback(cb_exit);
	when(FL_HIDE);
}

ctrlPanel::ctrlPanel(int h, const char* title) : Fl_Window(100, 100, title), widgets() 
{ 
	m_nTargetHeight=h; 
	m_nWidgetPerColumn=(m_nTargetHeight-INIT_SPACING)/YPIXELS_PER_WIDGET; 
	m_pObject=NULL;
	user_data(this);
	callback(cb_exit);
	when(FL_HIDE);
}

ctrlPanel::ctrlPanel(int x, int y, const char* title) : Fl_Window(x, y, 100, 100, title), widgets() 
{ 
	m_nTargetHeight=WINDOW_HEIGHT ;
	m_nWidgetPerColumn=(m_nTargetHeight-INIT_SPACING)/YPIXELS_PER_WIDGET; 
	m_pObject=NULL;
	user_data(this);
	callback(cb_exit);
	when(FL_HIDE);
}

ctrlPanel::ctrlPanel(int x, int y, int h, const char* title) : Fl_Window(x, y, 100, 100, title), widgets() 
{ 
	m_nTargetHeight=h; 
	m_nWidgetPerColumn=(m_nTargetHeight-INIT_SPACING)/YPIXELS_PER_WIDGET; 
	m_pObject=NULL;
	user_data(this);
	callback(cb_exit);
	when(FL_HIDE);
};

ctrlPanel::~ctrlPanel()
{
	// delete all children widgets
	while (children()!=0) {
		Fl_Widget *w=child(0);
		remove(w);
		delete w;
	}
	// delete pair in map
	map_iterator j;

	for( j = begin(); j != end(); ++j ) {
		delete (*j).second;
	}
}

void ctrlPanel::makeCurrent()
{
	g_panel = this;
}

void ctrlPanel::cb_exit(Fl_Widget* o, void* v) 
{	
	ctrlPanel* self=(ctrlPanel *)(o->user_data());

	((CAniUI*)(self->getUI()))->removePanel(self);
	self->hide();
}

void ctrlPanel::close()
{
	int n=children();

	int height=m_nTargetHeight;
	m_nWidgetPerColumn=(height-INIT_SPACING)/YPIXELS_PER_WIDGET;
	
	int cols=(n/m_nWidgetPerColumn);
	
	if (cols==0) {
		height=(n%m_nWidgetPerColumn)*YPIXELS_PER_WIDGET+2*INIT_SPACING;
	} else {
		height=m_nWidgetPerColumn*YPIXELS_PER_WIDGET+2*INIT_SPACING;
	}

	cols+=((n%m_nWidgetPerColumn)?1:0);
	int width=cols*XPIXELS_PER_WIDGET+2*INIT_SPACING;
	
	size(width, height);
}

void ctrlPanel::curve(int id, char *name, float min, float max, float start)
{
	values[id]=new value_pair(start, start, NULL, name, true);
}

void ctrlPanel::scale(int id, char *name, float min, float max, float res, float start)
{
	int n=children();

	Fl_Value_Slider *s=new Fl_Value_Slider((n/m_nWidgetPerColumn)*XPIXELS_PER_WIDGET+INIT_SPACING, 
										   (n%m_nWidgetPerColumn)*YPIXELS_PER_WIDGET+INIT_SPACING,
											SLIDER_WIDTH, WIDGET_HEIGHT, name);


	s->type(FL_HOR_NICE_SLIDER);
	s->labelfont(FL_COURIER);
    s->labelsize(FONT_SIZE);
	s->minimum(min);
	s->maximum(max);

	int A, B;
	if (res<1.0) {
		int f;
		if (res>0.1) f=10;
		else if (res>0.01) f=100;
		else if (res>0.001) f=1000;
		else f=10000;
		
		A=(int)(res*f);
		B=f;
		s->step(A, B);
	} else {
		s->step(res);
	}

	s->value(start);
	s->align(FL_ALIGN_RIGHT);
	s->callback(cb_slides);

	s->user_data(new params((void*)(this), id) );

	add(s);

	values[id]=new value_pair(start, start, NULL, name, true);
}

void ctrlPanel::cb_slides(Fl_Widget *o, void *v)
{
	params* args=(params *)(o->user_data());

	ctrlPanel *self=(ctrlPanel*)(args->p);
	int id=args->id;

	self->values[id]->value=((Fl_Value_Slider *)o)->value();

	if (self->m_pObject)
		self->m_pObject->refresh();
}

void ctrlPanel::checkbox(int id, char *name, int state)
{
	int n=children();

	Fl_Light_Button *b=new Fl_Light_Button((n/m_nWidgetPerColumn)*XPIXELS_PER_WIDGET+INIT_SPACING, 
										   (n%m_nWidgetPerColumn)*YPIXELS_PER_WIDGET+INIT_SPACING,
											CHECKBUTTON_WIDTH, WIDGET_HEIGHT, name);


	b->box(FL_UP_BOX);
	b->labelfont(FL_COURIER);
    b->labelsize(FONT_SIZE);
	b->value(state);
	b->align(FL_ALIGN_RIGHT);
	b->callback(cb_check);

	b->user_data(new params((void*)(this), id) );

	add(b);

	if (state) 	values[id]=new value_pair(1.0, 1.0, NULL, name, false);
	else values[id]=new value_pair(0.0, 0.0, NULL, name, false);
}

void ctrlPanel::cb_check(Fl_Widget *o, void *v)
{
	params* args=(params *)(o->user_data());

	ctrlPanel *self=(ctrlPanel*)(args->p);
	int id=args->id;

	if ((int)(self->values[id]->value)==0) self->values[id]->value=1;
	else self->values[id]->value=0;

	if (self->m_pObject)
		self->m_pObject->refresh();
}

void ctrlPanel::menu(int id, char *name, int start, Fl_Menu_Item* menu )
{
	int n=children();

	Fl_Choice *m=new Fl_Choice((n/m_nWidgetPerColumn)*XPIXELS_PER_WIDGET+INIT_SPACING, 
							   (n%m_nWidgetPerColumn)*YPIXELS_PER_WIDGET+INIT_SPACING,
							   MENU_WIDTH, WIDGET_HEIGHT, name);

	m->menu(menu);
	m->labelfont(FL_COURIER);
    m->labelsize(FONT_SIZE);
	m->value(start);
	m->align(FL_ALIGN_RIGHT);
	m->callback(cb_menu);	

	m->user_data(new params((void*)(this), id) );

	add(m);

	values[id]=new value_pair((float)start, (float)start, NULL, name, false);
}

void ctrlPanel::cb_menu(Fl_Widget *o, void *v)
{
	params* args=(params *)(o->user_data());

	ctrlPanel *self=(ctrlPanel*)(args->p);
	int id=args->id;

	self->values[id]->value=((Fl_Choice *)o)->value();

	if (self->m_pObject)
		self->m_pObject->refresh();
}

void ctrlPanel::ptr(int id, void *ptr )
{
	values[id]=new value_pair(0, 0, ptr, "", false);
}

ctrlPanel::map_iterator ctrlPanel::begin(void) const
{
	return values.begin();
}

ctrlPanel::map_iterator ctrlPanel::end(void) const
{
	return values.end();
}